package com.sun.java.util.jar.pack;

import java.io.FilterInputStream;
import java.io.BufferedInputStream;
import java.util.ListIterator;
import java.io.ByteArrayOutputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.EOFException;
import java.io.PrintStream;
import java.io.IOException;
import java.util.HashMap;
import java.io.InputStream;
import java.util.Set;
import java.util.List;
import java.util.Comparator;
import java.util.Map;

class PackageReader extends BandStructure
{
    Package pkg;
    byte[] bytes;
    PackageReader.LimitedBuffer in;
    Package.Version packageVersion;
    int[] tagCount;
    int numFiles;
    int numAttrDefs;
    int numInnerClasses;
    int numClasses;
    static final int MAGIC_BYTES = 4;
    Map<ConstantPool.Utf8Entry, ConstantPool.SignatureEntry> utf8Signatures;
    static final int NO_FLAGS_YET = 0;
    Comparator<ConstantPool.Entry> entryOutputOrder;
    Code[] allCodes;
    List<Code> codesWithFlags;
    Map<Package.Class, Set<ConstantPool.Entry>> ldcRefMap;
    
    PackageReader(Package pkg, InputStream inputStream) throws IOException {
        this.tagCount = new int[19];
        this.entryOutputOrder = new Comparator<ConstantPool.Entry>() {
            @Override
            public int compare(ConstantPool.Entry entry, ConstantPool.Entry entry2) {
                int access$000 = PackageReader.access$000(PackageReader.this, entry);
                int access$2 = PackageReader.access$000(PackageReader.this, entry2);
                if (access$000 >= 0 && access$2 >= 0) {
                    return access$000 - access$2;
                }
                if (access$000 == access$2) {
                    return entry.compareTo(entry2);
                }
                return (access$000 >= 0) ? -1 : 1;
            }
        };
        this.ldcRefMap = new HashMap();
        this.pkg = pkg;
        this.in = new LimitedBuffer(inputStream);
    }
    
    void read() throws IOException {
        try {
            this.readFileHeader();
            this.readBandHeaders();
            this.readConstantPool();
            this.readAttrDefs();
            this.readInnerClasses();
            Package.Class[] classes = this.readClasses();
            this.readByteCodes();
            this.readFiles();
            assert !(!this.in.atLimit());
            assert this.in.getBytesServed() == this.archiveSize0 + this.archiveSize1;
            this.all_bands.doneDisbursing();
            for (int i = 0; i < classes.length; ++i) {
                this.reconstructClass(classes[i]);
            }
        }
        catch (Exception ex) {
            Utils.log.warning("Error on input: " + ex, ex);
            if (this.verbose > 0) {
                Utils.log.info("Stream offsets: served=" + this.in.getBytesServed() + " buffered=" + this.in.buffered + " limit=" + this.in.limit);
            }
            if ((ex instanceof IOException)) {
                throw (IOException)ex;
            }
            if ((ex instanceof RuntimeException)) {
                throw (RuntimeException)ex;
            }
            throw new Error("error unpacking", ex);
        }
    }
    
    void readFileHeader() throws IOException {
        this.readArchiveMagic();
        this.readArchiveHeader();
    }
    
    private int getMagicInt32() throws IOException {
        int n = 0;
        for (int i = 0; i < 4; ++i) {
            n = (n << 8 | (this.archive_magic.getByte() & 0xFF));
        }
        return n;
    }
    
    void readArchiveMagic() throws IOException {
        this.in.setReadLimit(19L);
        this.archive_magic.expectLength(4);
        this.archive_magic.readFrom(this.in);
        int magicInt32 = this.getMagicInt32();
        this.pkg.getClass();
        if (-889270259 != magicInt32) {
            StringBuilder append = new StringBuilder().append("Unexpected package magic number: got ").append(magicInt32).append("; expected ");
            this.pkg.getClass();
            throw new IOException(append.append(-889270259).toString());
        }
        this.archive_magic.doneDisbursing();
        return;
    }
    
    void checkArchiveVersion() throws IOException {
        Package.Version version = null;
        Package.Version[] array = { Constants.JAVA8_PACKAGE_VERSION, Constants.JAVA7_PACKAGE_VERSION, Constants.JAVA6_PACKAGE_VERSION, Constants.JAVA5_PACKAGE_VERSION };
        int length = array.length;
        int i = 0;
        while (i < length) {
            Package.Version version2 = array[i];
            if (this.packageVersion.equals(version2)) {
                version = version2;
                break;
            }
            ++i;
        }
        if (version == null) {
            throw new IOException("Unexpected package minor version: got " + this.packageVersion.toString() + "; expected " + (Constants.JAVA8_PACKAGE_VERSION.toString() + "OR" + Constants.JAVA7_PACKAGE_VERSION.toString() + " OR " + Constants.JAVA6_PACKAGE_VERSION.toString() + " OR " + Constants.JAVA5_PACKAGE_VERSION.toString()));
        }
        return;
    }
    
    void readArchiveHeader() throws IOException {
        this.archive_header_0.expectLength(3);
        this.archive_header_0.readFrom(this.in);
        this.packageVersion = Package.Version.of(this.archive_header_0.getInt(), this.archive_header_0.getInt());
        this.checkArchiveVersion();
        this.initHighestClassVersion(Constants.JAVA7_MAX_CLASS_VERSION);
        this.archiveOptions = this.archive_header_0.getInt();
        this.archive_header_0.doneDisbursing();
        int testBit = BandStructure.testBit(this.archiveOptions, 1);
        int testBit2 = BandStructure.testBit(this.archiveOptions, 16);
        int testBit3 = BandStructure.testBit(this.archiveOptions, 2);
        int testBit4 = BandStructure.testBit(this.archiveOptions, 8);
        this.initAttrIndexLimit();
        this.archive_header_S.expectLength(testBit2 ? 2 : 0);
        this.archive_header_S.readFrom(this.in);
        if (!(testBit2 == 0)) {
            this.archiveSize1 = ((long)this.archive_header_S.getInt() << 32) + ((long)this.archive_header_S.getInt() << 32 >>> 32);
            this.in.setReadLimit(this.archiveSize1);
        }
        else {
            this.archiveSize1 = 0L;
            this.in.setReadLimit(-1L);
        }
        this.archive_header_S.doneDisbursing();
        this.archiveSize0 = this.in.getBytesServed();
        int n = 10;
        if (!(testBit2 == 0)) {
            n += 5;
        }
        if (!(testBit == 0)) {
            n += 2;
        }
        if (!(testBit3 == 0)) {
            n += 4;
        }
        if (!(testBit4 == 0)) {
            n += 4;
        }
        this.archive_header_1.expectLength(n);
        this.archive_header_1.readFrom(this.in);
        if (!(testBit2 == 0)) {
            this.archiveNextCount = this.archive_header_1.getInt();
            this.pkg.default_modtime = this.archive_header_1.getInt();
            this.numFiles = this.archive_header_1.getInt();
        }
        else {
            this.archiveNextCount = 0;
            this.numFiles = 0;
        }
        if (!(testBit == 0)) {
            this.band_headers.expectLength(this.archive_header_1.getInt());
            this.numAttrDefs = this.archive_header_1.getInt();
        }
        else {
            this.band_headers.expectLength(0);
            this.numAttrDefs = 0;
        }
        this.readConstantPoolCounts((boolean)testBit3, (boolean)testBit4);
        this.numInnerClasses = this.archive_header_1.getInt();
        this.pkg.defaultClassVersion = Package.Version.of((int)(short)this.archive_header_1.getInt(), (int)(short)this.archive_header_1.getInt());
        this.numClasses = this.archive_header_1.getInt();
        this.archive_header_1.doneDisbursing();
        if (BandStructure.testBit(this.archiveOptions, 32)) {
            Package pkg = this.pkg;
            pkg.default_options |= 0x1;
        }
    }
    
    void readBandHeaders() throws IOException {
        this.band_headers.readFrom(this.in);
        this.bandHeaderBytePos = 1;
        this.bandHeaderBytes = new byte[this.bandHeaderBytePos + this.band_headers.length()];
        for (int i = this.bandHeaderBytePos; i < this.bandHeaderBytes.length; ++i) {
            this.bandHeaderBytes[i] = (byte)this.band_headers.getByte();
        }
        this.band_headers.doneDisbursing();
    }
    
    void readConstantPoolCounts(boolean b, boolean b2) throws IOException {
        for (int i = 0; i < ConstantPool.TAGS_IN_ORDER.length; ++i) {
            int n = ConstantPool.TAGS_IN_ORDER[i];
            if (!b) {
                switch (n) {
                    case 3:
                    case 4:
                    case 5:
                    case 6: {
                        continue;
                    }
                    default: {
                        break;
                    }
                }
            }
            if (!b2) {
                switch (n) {
                    case 15:
                    case 16:
                    case 17:
                    case 18: {
                        continue;
                    }
                    default: {
                        break;
                    }
                }
            }
            this.tagCount[n] = this.archive_header_1.getInt();
        }
    }
    
    @Override
    protected ConstantPool.Index getCPIndex(byte b) {
        return this.pkg.cp.getIndexByTag(b);
    }
    
    ConstantPool.Index initCPIndex(byte b, ConstantPool.Entry[] array) {
        if (this.verbose > 3) {
            for (int i = 0; i < array.length; ++i) {
                Utils.log.fine("cp.add " + array[i]);
            }
        }
        ConstantPool.Index index = ConstantPool.makeIndex(ConstantPool.tagName((int)b), array);
        if (this.verbose > 1) {
            Utils.log.fine("Read " + index);
        }
        this.pkg.cp.initIndexByTag(b, index);
        return index;
    }
    
    void checkLegacy(String str) {
        if (this.packageVersion.lessThan(Constants.JAVA7_PACKAGE_VERSION)) {
            throw new RuntimeException("unexpected band " + str);
        }
        return;
    }
    
    void readConstantPool() throws IOException {
        if (this.verbose > 0) {
            Utils.log.info("Reading CP");
        }
        for (int i = 0; i < ConstantPool.TAGS_IN_ORDER.length; ++i) {
            int n = ConstantPool.TAGS_IN_ORDER[i];
            ConstantPool.Entry[] array = new ConstantPool.Entry[this.tagCount[n]];
            if (this.verbose > 0) {
                Utils.log.info("Reading " + array.length + " " + ConstantPool.tagName(n) + " entries...");
            }
            switch (n) {
                case 1: {
                    this.readUtf8Bands(array);
                    break;
                }
                case 3: {
                    this.cp_Int.expectLength(array.length);
                    this.cp_Int.readFrom(this.in);
                    for (int j = 0; j < array.length; ++j) {
                        array[j] = ConstantPool.getLiteralEntry(Integer.valueOf(this.cp_Int.getInt()));
                    }
                    this.cp_Int.doneDisbursing();
                    break;
                }
                case 4: {
                    this.cp_Float.expectLength(array.length);
                    this.cp_Float.readFrom(this.in);
                    for (int k = 0; k < array.length; ++k) {
                        array[k] = ConstantPool.getLiteralEntry(Float.valueOf(Float.intBitsToFloat(this.cp_Float.getInt())));
                    }
                    this.cp_Float.doneDisbursing();
                    break;
                }
                case 5: {
                    this.cp_Long_hi.expectLength(array.length);
                    this.cp_Long_hi.readFrom(this.in);
                    this.cp_Long_lo.expectLength(array.length);
                    this.cp_Long_lo.readFrom(this.in);
                    for (int l = 0; l < array.length; ++l) {
                        array[l] = ConstantPool.getLiteralEntry(Long.valueOf(((long)this.cp_Long_hi.getInt() << 32) + ((long)this.cp_Long_lo.getInt() << 32 >>> 32)));
                    }
                    this.cp_Long_hi.doneDisbursing();
                    this.cp_Long_lo.doneDisbursing();
                    break;
                }
                case 6: {
                    this.cp_Double_hi.expectLength(array.length);
                    this.cp_Double_hi.readFrom(this.in);
                    this.cp_Double_lo.expectLength(array.length);
                    this.cp_Double_lo.readFrom(this.in);
                    for (int n2 = 0; n2 < array.length; ++n2) {
                        array[n2] = ConstantPool.getLiteralEntry(Double.valueOf(Double.longBitsToDouble(((long)this.cp_Double_hi.getInt() << 32) + ((long)this.cp_Double_lo.getInt() << 32 >>> 32))));
                    }
                    this.cp_Double_hi.doneDisbursing();
                    this.cp_Double_lo.doneDisbursing();
                    break;
                }
                case 8: {
                    this.cp_String.expectLength(array.length);
                    this.cp_String.readFrom(this.in);
                    this.cp_String.setIndex(this.getCPIndex((byte)1));
                    for (int n3 = 0; n3 < array.length; ++n3) {
                        array[n3] = ConstantPool.getLiteralEntry(this.cp_String.getRef().stringValue());
                    }
                    this.cp_String.doneDisbursing();
                    break;
                }
                case 7: {
                    this.cp_Class.expectLength(array.length);
                    this.cp_Class.readFrom(this.in);
                    this.cp_Class.setIndex(this.getCPIndex((byte)1));
                    for (int n4 = 0; n4 < array.length; ++n4) {
                        array[n4] = ConstantPool.getClassEntry(this.cp_Class.getRef().stringValue());
                    }
                    this.cp_Class.doneDisbursing();
                    break;
                }
                case 13: {
                    this.readSignatureBands(array);
                    break;
                }
                case 12: {
                    this.cp_Descr_name.expectLength(array.length);
                    this.cp_Descr_name.readFrom(this.in);
                    this.cp_Descr_name.setIndex(this.getCPIndex((byte)1));
                    this.cp_Descr_type.expectLength(array.length);
                    this.cp_Descr_type.readFrom(this.in);
                    this.cp_Descr_type.setIndex(this.getCPIndex((byte)13));
                    for (int n5 = 0; n5 < array.length; ++n5) {
                        array[n5] = ConstantPool.getDescriptorEntry((ConstantPool.Utf8Entry)this.cp_Descr_name.getRef(), (ConstantPool.SignatureEntry)this.cp_Descr_type.getRef());
                    }
                    this.cp_Descr_name.doneDisbursing();
                    this.cp_Descr_type.doneDisbursing();
                    break;
                }
                case 9: {
                    this.readMemberRefs((byte)n, array, this.cp_Field_class, this.cp_Field_desc);
                    break;
                }
                case 10: {
                    this.readMemberRefs((byte)n, array, this.cp_Method_class, this.cp_Method_desc);
                    break;
                }
                case 11: {
                    this.readMemberRefs((byte)n, array, this.cp_Imethod_class, this.cp_Imethod_desc);
                    break;
                }
                case 15: {
                    if (array.length > 0) {
                        this.checkLegacy(this.cp_MethodHandle_refkind.name());
                    }
                    this.cp_MethodHandle_refkind.expectLength(array.length);
                    this.cp_MethodHandle_refkind.readFrom(this.in);
                    this.cp_MethodHandle_member.expectLength(array.length);
                    this.cp_MethodHandle_member.readFrom(this.in);
                    this.cp_MethodHandle_member.setIndex(this.getCPIndex((byte)52));
                    for (int n6 = 0; n6 < array.length; ++n6) {
                        array[n6] = ConstantPool.getMethodHandleEntry((byte)this.cp_MethodHandle_refkind.getInt(), (ConstantPool.MemberEntry)this.cp_MethodHandle_member.getRef());
                    }
                    this.cp_MethodHandle_refkind.doneDisbursing();
                    this.cp_MethodHandle_member.doneDisbursing();
                    break;
                }
                case 16: {
                    if (array.length > 0) {
                        this.checkLegacy(this.cp_MethodType.name());
                    }
                    this.cp_MethodType.expectLength(array.length);
                    this.cp_MethodType.readFrom(this.in);
                    this.cp_MethodType.setIndex(this.getCPIndex((byte)13));
                    for (int n7 = 0; n7 < array.length; ++n7) {
                        array[n7] = ConstantPool.getMethodTypeEntry((ConstantPool.SignatureEntry)this.cp_MethodType.getRef());
                    }
                    this.cp_MethodType.doneDisbursing();
                    break;
                }
                case 18: {
                    if (array.length > 0) {
                        this.checkLegacy(this.cp_InvokeDynamic_spec.name());
                    }
                    this.cp_InvokeDynamic_spec.expectLength(array.length);
                    this.cp_InvokeDynamic_spec.readFrom(this.in);
                    this.cp_InvokeDynamic_spec.setIndex(this.getCPIndex((byte)17));
                    this.cp_InvokeDynamic_desc.expectLength(array.length);
                    this.cp_InvokeDynamic_desc.readFrom(this.in);
                    this.cp_InvokeDynamic_desc.setIndex(this.getCPIndex((byte)12));
                    for (int n8 = 0; n8 < array.length; ++n8) {
                        array[n8] = ConstantPool.getInvokeDynamicEntry((ConstantPool.BootstrapMethodEntry)this.cp_InvokeDynamic_spec.getRef(), (ConstantPool.DescriptorEntry)this.cp_InvokeDynamic_desc.getRef());
                    }
                    this.cp_InvokeDynamic_spec.doneDisbursing();
                    this.cp_InvokeDynamic_desc.doneDisbursing();
                    break;
                }
                case 17: {
                    if (array.length > 0) {
                        this.checkLegacy(this.cp_BootstrapMethod_ref.name());
                    }
                    this.cp_BootstrapMethod_ref.expectLength(array.length);
                    this.cp_BootstrapMethod_ref.readFrom(this.in);
                    this.cp_BootstrapMethod_ref.setIndex(this.getCPIndex((byte)15));
                    this.cp_BootstrapMethod_arg_count.expectLength(array.length);
                    this.cp_BootstrapMethod_arg_count.readFrom(this.in);
                    this.cp_BootstrapMethod_arg.expectLength(this.cp_BootstrapMethod_arg_count.getIntTotal());
                    this.cp_BootstrapMethod_arg.readFrom(this.in);
                    this.cp_BootstrapMethod_arg.setIndex(this.getCPIndex((byte)51));
                    ConstantPool.MethodHandleEntry methodHandleEntry;
                    ConstantPool.Entry[] array2;
                    for (int n9 = 0; n9 < array.length; array[n9] = ConstantPool.getBootstrapMethodEntry(methodHandleEntry, array2), ++n9) {
                        methodHandleEntry = (ConstantPool.MethodHandleEntry)this.cp_BootstrapMethod_ref.getRef();
                        int int1 = this.cp_BootstrapMethod_arg_count.getInt();
                        array2 = new ConstantPool.Entry[int1];
                        for (int n10 = 0; n10 < int1; ++n10) {
                            array2[n10] = this.cp_BootstrapMethod_arg.getRef();
                        }
                    }
                    this.cp_BootstrapMethod_ref.doneDisbursing();
                    this.cp_BootstrapMethod_arg_count.doneDisbursing();
                    this.cp_BootstrapMethod_arg.doneDisbursing();
                    break;
                }
                default: {
                    throw new AssertionError("unexpected CP tag in package");
                }
            }
            ConstantPool.Index initCPIndex = this.initCPIndex((byte)n, array);
            if (this.optDumpBands) {
                try (final Object o = new PrintStream(BandStructure.getDumpStream(initCPIndex, ".idx"))) {
                    BandStructure.printArrayTo(o, initCPIndex.cpMap, 0, initCPIndex.cpMap.length);
                }
            }
        }
        this.cp_bands.doneDisbursing();
        if (this.optDumpBands || this.verbose > 1) {
            for (int n11 = 50; n11 < 54; n11 = (byte)(n11 + 1)) {
                ConstantPool.Index indexByTag = this.pkg.cp.getIndexByTag((byte)n11);
                if (indexByTag != null && !indexByTag.isEmpty()) {
                    ConstantPool.Entry[] cpMap = indexByTag.cpMap;
                    if (this.verbose > 1) {
                        Utils.log.info("Index group " + ConstantPool.tagName(n11) + " contains " + cpMap.length + " entries.");
                    }
                    if (this.optDumpBands) {
                        try (final Object o2 = new PrintStream(BandStructure.getDumpStream(indexByTag.debugName, n11, ".gidx", indexByTag))) {
                            BandStructure.printArrayTo(o2, cpMap, 0, cpMap.length, true);
                        }
                    }
                }
            }
        }
        this.setBandIndexes();
    }
    
    void readUtf8Bands(ConstantPool.Entry[] array) throws IOException {
        int length = array.length;
        if (length == 0) {
            return;
        }
        this.cp_Utf8_prefix.expectLength(Math.max(0, length - 2));
        this.cp_Utf8_prefix.readFrom(this.in);
        this.cp_Utf8_suffix.expectLength(Math.max(0, length - 1));
        this.cp_Utf8_suffix.readFrom(this.in);
        char[][] array2 = new char[length][];
        int n = 0;
        this.cp_Utf8_chars.expectLength(this.cp_Utf8_suffix.getIntTotal());
        this.cp_Utf8_chars.readFrom(this.in);
        for (int i = 0; i < length; ++i) {
            int n2 = (i < 1) ? 0 : this.cp_Utf8_suffix.getInt();
            if (n2 == 0 && i >= 1) {
                ++n;
            }
            else {
                array2[i] = new char[n2];
                int j = 0;
                while (j < n2) {
                    int int1 = this.cp_Utf8_chars.getInt();
                    assert int1 == (char)int1;
                    array2[i][j] = (char)int1;
                    ++j;
                }
            }
        }
        this.cp_Utf8_chars.doneDisbursing();
        int n3 = 0;
        this.cp_Utf8_big_suffix.expectLength(n);
        this.cp_Utf8_big_suffix.readFrom(this.in);
        this.cp_Utf8_suffix.resetForSecondPass();
        for (int k = 0; k < length; ++k) {
            int int2 = (k < 1) ? 0 : this.cp_Utf8_suffix.getInt();
            int n4 = (k < 2) ? 0 : this.cp_Utf8_prefix.getInt();
            if (int2 == 0 && k >= 1) {
                assert array2[k] == null;
                int2 = this.cp_Utf8_big_suffix.getInt();
            }
            else
                assert array2[k] != null;
            if (n3 < n4 + int2) {
                n3 = n4 + int2;
            }
        }
        char[] value = new char[n3];
        this.cp_Utf8_suffix.resetForSecondPass();
        this.cp_Utf8_big_suffix.resetForSecondPass();
        for (int l = 0; l < length; ++l) {
            if (l >= 1 && this.cp_Utf8_suffix.getInt() == 0) {
                int int3 = this.cp_Utf8_big_suffix.getInt();
                array2[l] = new char[int3];
                if (int3 != 0) {
                    BandStructure.IntBand intBand = this.cp_Utf8_big_chars.newIntBand("(Utf8_big_" + l + ")");
                    intBand.expectLength(int3);
                    intBand.readFrom(this.in);
                    int n5 = 0;
                    while (n5 < int3) {
                        int int4 = intBand.getInt();
                        assert int4 == (char)int4;
                        array2[l][n5] = (char)int4;
                        ++n5;
                    }
                    intBand.doneDisbursing();
                }
            }
        }
        this.cp_Utf8_big_chars.doneDisbursing();
        this.cp_Utf8_prefix.resetForSecondPass();
        this.cp_Utf8_suffix.resetForSecondPass();
        this.cp_Utf8_big_suffix.resetForSecondPass();
        int n7;
        int int5;
        for (int n6 = 0; n6 < length; array[n6] = ConstantPool.getUtf8Entry(new String(value, 0, n7 + int5)), ++n6) {
            n7 = ((n6 < 2) ? 0 : this.cp_Utf8_prefix.getInt());
            int5 = ((n6 < 1) ? 0 : this.cp_Utf8_suffix.getInt());
            if (int5 == 0 && n6 >= 1) {
                int5 = this.cp_Utf8_big_suffix.getInt();
            }
            System.arraycopy(array2[n6], 0, value, n7, int5);
        }
        this.cp_Utf8_prefix.doneDisbursing();
        this.cp_Utf8_suffix.doneDisbursing();
        this.cp_Utf8_big_suffix.doneDisbursing();
        return;
    }
    
    void readSignatureBands(ConstantPool.Entry[] array) throws IOException {
        this.cp_Signature_form.expectLength(array.length);
        this.cp_Signature_form.readFrom(this.in);
        this.cp_Signature_form.setIndex(this.getCPIndex((byte)1));
        int[] array2 = new int[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = ConstantPool.countClassParts((ConstantPool.Utf8Entry)this.cp_Signature_form.getRef());
        }
        this.cp_Signature_form.resetForSecondPass();
        this.cp_Signature_classes.expectLength(BandStructure.getIntTotal(array2));
        this.cp_Signature_classes.readFrom(this.in);
        this.cp_Signature_classes.setIndex(this.getCPIndex((byte)7));
        this.utf8Signatures = new HashMap();
        for (int j = 0; j < array.length; ++j) {
            ConstantPool.Utf8Entry utf8Entry = (ConstantPool.Utf8Entry)this.cp_Signature_form.getRef();
            ConstantPool.ClassEntry[] array3 = new ConstantPool.ClassEntry[array2[j]];
            for (int k = 0; k < array3.length; ++k) {
                array3[k] = (ConstantPool.ClassEntry)this.cp_Signature_classes.getRef();
            }
            ConstantPool.SignatureEntry signatureEntry = ConstantPool.getSignatureEntry(utf8Entry, array3);
            array[j] = signatureEntry;
            this.utf8Signatures.put(signatureEntry.asUtf8Entry(), signatureEntry);
        }
        this.cp_Signature_form.doneDisbursing();
        this.cp_Signature_classes.doneDisbursing();
    }
    
    void readMemberRefs(byte b, ConstantPool.Entry[] array, BandStructure.CPRefBand cpRefBand, BandStructure.CPRefBand cpRefBand2) throws IOException {
        cpRefBand.expectLength(array.length);
        cpRefBand.readFrom(this.in);
        cpRefBand.setIndex(this.getCPIndex((byte)7));
        cpRefBand2.expectLength(array.length);
        cpRefBand2.readFrom(this.in);
        cpRefBand2.setIndex(this.getCPIndex((byte)12));
        for (int i = 0; i < array.length; ++i) {
            array[i] = ConstantPool.getMemberEntry(b, (ConstantPool.ClassEntry)cpRefBand.getRef(), (ConstantPool.DescriptorEntry)cpRefBand2.getRef());
        }
        cpRefBand.doneDisbursing();
        cpRefBand2.doneDisbursing();
    }
    
    void readFiles() throws IOException {
        if (this.verbose > 0) {
            Utils.log.info("  ...building " + this.numFiles + " files...");
        }
        this.file_name.expectLength(this.numFiles);
        this.file_size_lo.expectLength(this.numFiles);
        int archiveOptions = this.archiveOptions;
        int testBit = BandStructure.testBit(archiveOptions, 256);
        int testBit2 = BandStructure.testBit(archiveOptions, 64);
        int testBit3 = BandStructure.testBit(archiveOptions, 128);
        if (!(testBit == 0)) {
            this.file_size_hi.expectLength(this.numFiles);
        }
        if (!(testBit2 == 0)) {
            this.file_modtime.expectLength(this.numFiles);
        }
        if (!(testBit3 == 0)) {
            this.file_options.expectLength(this.numFiles);
        }
        this.file_name.readFrom(this.in);
        this.file_size_hi.readFrom(this.in);
        this.file_size_lo.readFrom(this.in);
        this.file_modtime.readFrom(this.in);
        this.file_options.readFrom(this.in);
        this.file_bits.setInputStreamFrom(this.in);
        Iterator<Package.Class> iterator = this.pkg.getClasses().iterator();
        long n = 0L;
        long[] array = new long[this.numFiles];
        long n2;
        for (int i = 0; i < this.numFiles; array[i] = n2, n += n2, ++i) {
            n2 = (long)this.file_size_lo.getInt() << 32 >>> 32;
            if (!(testBit == 0)) {
                n2 += (long)this.file_size_hi.getInt() << 32;
            }
        }
        assert this.in.getReadLimit() == n;
        byte[] b = new byte[65536];
        for (int j = 0; j < this.numFiles; ++j) {
            ConstantPool.Utf8Entry utf8Entry = (ConstantPool.Utf8Entry)this.file_name.getRef();
            long lng = array[j];
            Package.File o = this.pkg.new Package.File(utf8Entry);
            o.modtime = this.pkg.default_modtime;
            o.options = this.pkg.default_options;
            if (!(testBit2 == 0)) {
                o.modtime += this.file_modtime.getInt();
            }
            if (!(testBit3 == 0)) {
                o.options |= this.file_options.getInt();
            }
            if (this.verbose > 1) {
                Utils.log.fine("Reading " + lng + " bytes of " + utf8Entry.stringValue());
            }
            while (lng > 0L) {
                int length = b.length;
                if ((long)length > lng) {
                    length = (int)lng;
                }
                int read = this.file_bits.getInputStream().read(b, 0, length);
                if (read < 0) {
                    throw new EOFException();
                }
                o.addBytes(b, 0, read);
                lng -= (long)read;
            }
            this.pkg.addFile(o);
            if (o.isClassStub()) {
                assert o.getFileLength() == 0L;
                ((Package.Class)iterator.next()).initFile(o);
            }
        }
        while (iterator.hasNext()) {
            Package.Class class1 = (Package.Class)iterator.next();
            class1.initFile(null);
            class1.file.modtime = this.pkg.default_modtime;
        }
        this.file_name.doneDisbursing();
        this.file_size_hi.doneDisbursing();
        this.file_size_lo.doneDisbursing();
        this.file_modtime.doneDisbursing();
        this.file_options.doneDisbursing();
        this.file_bits.doneDisbursing();
        this.file_bands.doneDisbursing();
        if (this.archiveSize1 != 0L && !this.in.atLimit()) {
            throw new RuntimeException("Predicted archive_size " + this.archiveSize1 + " != " + (this.in.getBytesServed() - this.archiveSize0));
        }
        return;
    }
    
    void readAttrDefs() throws IOException {
        this.attr_definition_headers.expectLength(this.numAttrDefs);
        this.attr_definition_name.expectLength(this.numAttrDefs);
        this.attr_definition_layout.expectLength(this.numAttrDefs);
        this.attr_definition_headers.readFrom(this.in);
        this.attr_definition_name.readFrom(this.in);
        this.attr_definition_layout.readFrom(this.in);
        try (final PrintStream printStream = this.optDumpBands ? new PrintStream(BandStructure.getDumpStream(this.attr_definition_headers, ".def")) : null) {
            int i = 0;
            while (i < this.numAttrDefs) {
                int byte1 = this.attr_definition_headers.getByte();
                ConstantPool.Utf8Entry utf8Entry = (ConstantPool.Utf8Entry)this.attr_definition_name.getRef();
                ConstantPool.Utf8Entry utf8Entry2 = (ConstantPool.Utf8Entry)this.attr_definition_layout.getRef();
                int n = byte1 & 0x3;
                int j = (byte1 >> 2) - 1;
                Attribute.Layout obj = new Attribute.Layout(n, utf8Entry.stringValue(), utf8Entry2.stringValue());
                if (!obj.layoutForClassVersion(this.getHighestClassVersion()).equals(obj.layout())) {
                    throw new IOException("Bad attribute layout in archive: " + obj.layout());
                }
                this.setAttributeLayoutIndex(obj, j);
                if (printStream != null) {
                    printStream.println(j + " " + obj);
                }
                ++i;
            }
        }
        this.attr_definition_headers.doneDisbursing();
        this.attr_definition_name.doneDisbursing();
        this.attr_definition_layout.doneDisbursing();
        this.makeNewAttributeBands();
        this.attr_definition_bands.doneDisbursing();
    }
    
    void readInnerClasses() throws IOException {
        this.ic_this_class.expectLength(this.numInnerClasses);
        this.ic_this_class.readFrom(this.in);
        this.ic_flags.expectLength(this.numInnerClasses);
        this.ic_flags.readFrom(this.in);
        int n = 0;
        for (int i = 0; i < this.numInnerClasses; ++i) {
            if ((this.ic_flags.getInt() & 0x10000) != 0x0) {
                ++n;
            }
        }
        this.ic_outer_class.expectLength(n);
        this.ic_outer_class.readFrom(this.in);
        this.ic_name.expectLength(n);
        this.ic_name.readFrom(this.in);
        this.ic_flags.resetForSecondPass();
        ArrayList allInnerClasses = new ArrayList(this.numInnerClasses);
        int j = 0;
        while (j < this.numInnerClasses) {
            int int1 = this.ic_flags.getInt();
            int n2 = (int1 & 0x10000) != 0x0;
            int n3 = int1 & 0xFFFEFFFF;
            ConstantPool.ClassEntry classEntry = (ConstantPool.ClassEntry)this.ic_this_class.getRef();
            ConstantPool.ClassEntry classEntry2;
            ConstantPool.Utf8Entry utf8Entry;
            if (!(n2 == 0)) {
                classEntry2 = (ConstantPool.ClassEntry)this.ic_outer_class.getRef();
                utf8Entry = (ConstantPool.Utf8Entry)this.ic_name.getRef();
            }
            else {
                String[] innerClassName = Package.parseInnerClassName(classEntry.stringValue());
                assert innerClassName != null;
                String s = innerClassName[0];
                String s2 = innerClassName[2];
                if (s == null) {
                    classEntry2 = null;
                }
                else {
                    classEntry2 = ConstantPool.getClassEntry(s);
                }
                if (s2 == null) {
                    utf8Entry = null;
                }
                else {
                    utf8Entry = ConstantPool.getUtf8Entry(s2);
                }
            }
            Package.InnerClass o = new Package.InnerClass(classEntry, classEntry2, utf8Entry, n3);
            assert !(!o.predictable);
            allInnerClasses.add(o);
            ++j;
        }
        this.ic_flags.doneDisbursing();
        this.ic_this_class.doneDisbursing();
        this.ic_outer_class.doneDisbursing();
        this.ic_name.doneDisbursing();
        this.pkg.setAllInnerClasses(allInnerClasses);
        this.ic_bands.doneDisbursing();
    }
    
    void readLocalInnerClasses(Package.Class class1) throws IOException {
        int int1 = this.class_InnerClasses_N.getInt();
        ArrayList innerClasses = new ArrayList(int1);
        for (int i = 0; i < int1; ++i) {
            ConstantPool.ClassEntry classEntry = (ConstantPool.ClassEntry)this.class_InnerClasses_RC.getRef();
            int int2 = this.class_InnerClasses_F.getInt();
            if (int2 == 0) {
                Package.InnerClass globalInnerClass = this.pkg.getGlobalInnerClass(classEntry);
                assert globalInnerClass != null;
                innerClasses.add(globalInnerClass);
            }
            else {
                if (int2 == 65536) {
                    int2 = 0;
                }
                innerClasses.add(new Package.InnerClass(classEntry, (ConstantPool.ClassEntry)this.class_InnerClasses_outer_RCN.getRef(), (ConstantPool.Utf8Entry)this.class_InnerClasses_name_RUN.getRef(), int2));
            }
        }
        class1.setInnerClasses(innerClasses);
    }
    
    Package.Class[] readClasses() throws IOException {
        Package.Class[] a = new Package.Class[this.numClasses];
        if (this.verbose > 0) {
            Utils.log.info("  ...building " + a.length + " classes...");
        }
        this.class_this.expectLength(this.numClasses);
        this.class_super.expectLength(this.numClasses);
        this.class_interface_count.expectLength(this.numClasses);
        this.class_this.readFrom(this.in);
        this.class_super.readFrom(this.in);
        this.class_interface_count.readFrom(this.in);
        this.class_interface.expectLength(this.class_interface_count.getIntTotal());
        this.class_interface.readFrom(this.in);
        ConstantPool.ClassEntry classEntry;
        ConstantPool.ClassEntry classEntry2;
        ConstantPool.ClassEntry[] array;
        for (int i = 0; i < a.length; a[i] = this.pkg.new Package.Class(0, classEntry, classEntry2, array), ++i) {
            classEntry = (ConstantPool.ClassEntry)this.class_this.getRef();
            classEntry2 = (ConstantPool.ClassEntry)this.class_super.getRef();
            array = new ConstantPool.ClassEntry[this.class_interface_count.getInt()];
            for (int j = 0; j < array.length; ++j) {
                array[j] = (ConstantPool.ClassEntry)this.class_interface.getRef();
            }
            if (classEntry2 == classEntry) {
                classEntry2 = null;
            }
        }
        this.class_this.doneDisbursing();
        this.class_super.doneDisbursing();
        this.class_interface_count.doneDisbursing();
        this.class_interface.doneDisbursing();
        this.readMembers(a);
        this.countAndReadAttrs(0, Arrays.asList(a));
        this.pkg.trimToSize();
        this.readCodeHeaders();
        return a;
    }
    
    private int getOutputIndex(ConstantPool.Entry entry) {
        assert entry.tag != 13;
        int untypedIndex = this.pkg.cp.untypedIndexOf(entry);
        if (untypedIndex >= 0) {
            return untypedIndex;
        }
        if (entry.tag == 1) {
            return this.pkg.cp.untypedIndexOf((ConstantPool.Entry)this.utf8Signatures.get(entry));
        }
        return -1;
    }
    
    void reconstructClass(Package.Class obj) {
        if (this.verbose > 1) {
            Utils.log.fine("reconstruct " + obj);
        }
        Attribute attribute = obj.getAttribute(this.attrClassFileVersion);
        if (attribute != null) {
            obj.removeAttribute(attribute);
            obj.version = this.parseClassFileVersionAttr(attribute);
        }
        else {
            obj.version = this.pkg.defaultClassVersion;
        }
        obj.expandSourceFile();
        obj.setCPMap(this.reconstructLocalCPMap(obj));
    }
    
    ConstantPool.Entry[] reconstructLocalCPMap(Package.Class class1) {
        Set<Object> set = (Set)this.ldcRefMap.get(class1);
        HashSet set2 = new HashSet();
        class1.visitRefs(0, set2);
        ArrayList list = new ArrayList();
        class1.addAttribute(Package.attrBootstrapMethodsEmpty.canonicalInstance());
        ConstantPool.completeReferencesIn(set2, true, list);
        int expandLocalICs = class1.expandLocalICs();
        if (expandLocalICs != 0) {
            if (expandLocalICs > 0) {
                class1.visitInnerClassRefs(0, set2);
            }
            else {
                set2.clear();
                class1.visitRefs(0, set2);
            }
            ConstantPool.completeReferencesIn(set2, true, list);
        }
        if (list.isEmpty()) {
            class1.attributes.remove(Package.attrBootstrapMethodsEmpty.canonicalInstance());
        }
        else {
            set2.add(Package.getRefString("BootstrapMethods"));
            Collections.sort(list);
            class1.setBootstrapMethods(list);
        }
        int n = 0;
        Iterator<Object> iterator = set2.iterator();
        while (iterator.hasNext()) {
            if (((ConstantPool.Entry)iterator.next()).isDoubleWord()) {
                ++n;
            }
            else {
                continue;
            }
        }
        ConstantPool.Entry[] array = new ConstantPool.Entry[1 + n + set2.size()];
        int n2 = 1;
        if (set != null) {
            assert !(!set2.containsAll(set));
            Iterator<Object> iterator2 = set.iterator();
            while (iterator2.hasNext()) {
                array[n2++] = (ConstantPool.Entry)iterator2.next();
            }
            assert n2 == 1 + set.size();
            set2.removeAll(set);
        }
        Iterator<Object> iterator3 = set2.iterator();
        while (iterator3.hasNext()) {
            array[n2++] = (ConstantPool.Entry)iterator3.next();
        }
        assert n2 == n2 + set2.size();
        Arrays.sort(array, 1, n2, this.entryOutputOrder);
        Arrays.sort(array, n2, n2, this.entryOutputOrder);
        if (this.verbose > 3) {
            Utils.log.fine("CP of " + this + " {");
            for (ConstantPool.Entry obj : array) {
                Utils.log.fine("  " + ((obj == null) ? -1 : this.getOutputIndex(obj)) + " : " + obj);
            }
            Utils.log.fine("}");
        }
        int length;
        ConstantPool.Entry entry;
        for (length = array.length, ; --n2 >= 1; array[--length] = entry) {
            entry = array[n2];
            if (entry.isDoubleWord()) {
                array[--length] = null;
            }
        }
        assert length == 1;
        return array;
    }
    
    void readMembers(Package.Class[] array) throws IOException {
        assert array.length == this.numClasses;
        this.class_field_count.expectLength(this.numClasses);
        this.class_method_count.expectLength(this.numClasses);
        this.class_field_count.readFrom(this.in);
        this.class_method_count.readFrom(this.in);
        int intTotal = this.class_field_count.getIntTotal();
        int intTotal2 = this.class_method_count.getIntTotal();
        this.field_descr.expectLength(intTotal);
        this.method_descr.expectLength(intTotal2);
        if (this.verbose > 1) {
            Utils.log.fine("expecting #fields=" + intTotal + " and #methods=" + intTotal2 + " in #classes=" + this.numClasses);
        }
        ArrayList list = new ArrayList(intTotal);
        this.field_descr.readFrom(this.in);
        for (int i = 0; i < array.length; ++i) {
            Package.Class class1 = array[i];
            for (int int1 = this.class_field_count.getInt(), j = 0; j < int1; ++j) {
                list.add(class1.new Field(0, (ConstantPool.DescriptorEntry)this.field_descr.getRef()));
            }
        }
        this.class_field_count.doneDisbursing();
        this.field_descr.doneDisbursing();
        this.countAndReadAttrs(1, list);
        ArrayList o = new ArrayList(intTotal2);
        this.method_descr.readFrom(this.in);
        for (int k = 0; k < array.length; ++k) {
            Package.Class class2 = array[k];
            for (int int2 = this.class_method_count.getInt(), l = 0; l < int2; ++l) {
                o.add(class2.new Method(0, (ConstantPool.DescriptorEntry)this.method_descr.getRef()));
            }
        }
        this.class_method_count.doneDisbursing();
        this.method_descr.doneDisbursing();
        this.countAndReadAttrs(2, o);
        this.allCodes = this.buildCodeAttrs(o);
        return;
    }
    
    Code[] buildCodeAttrs(List<Package.Class.Method> list) {
        ArrayList list2 = new ArrayList(list.size());
        for (Package.Class.Method method : list) {
            if (method.getAttribute(this.attrCodeEmpty) != null) {
                list2.add(method.code = new Code(method));
            }
            else {
                continue;
            }
        }
        Code[] array = new Code[list2.size()];
        list2.toArray(array);
        return array;
    }
    
    void readCodeHeaders() throws IOException {
        int testBit = BandStructure.testBit(this.archiveOptions, 4);
        this.code_headers.expectLength(this.allCodes.length);
        this.code_headers.readFrom(this.in);
        ArrayList<Code> codesWithFlags = new ArrayList(this.allCodes.length / 10);
        int i = 0;
        while (i < this.allCodes.length) {
            Code obj = this.allCodes[i];
            int byte1 = this.code_headers.getByte();
            assert byte1 == (byte1 & 0xFF);
            if (this.verbose > 2) {
                Utils.log.fine("codeHeader " + obj + " = " + byte1);
            }
            if (byte1 == 0) {
                codesWithFlags.add(obj);
            }
            else {
                obj.setMaxStack(BandStructure.shortCodeHeader_max_stack(byte1));
                obj.setMaxNALocals(BandStructure.shortCodeHeader_max_na_locals(byte1));
                obj.setHandlerCount(BandStructure.shortCodeHeader_handler_count(byte1));
                assert BandStructure.shortCodeHeader(obj) == byte1;
            }
            ++i;
        }
        this.code_headers.doneDisbursing();
        this.code_max_stack.expectLength(codesWithFlags.size());
        this.code_max_na_locals.expectLength(codesWithFlags.size());
        this.code_handler_count.expectLength(codesWithFlags.size());
        this.code_max_stack.readFrom(this.in);
        this.code_max_na_locals.readFrom(this.in);
        this.code_handler_count.readFrom(this.in);
        for (Code code : codesWithFlags) {
            code.setMaxStack(this.code_max_stack.getInt());
            code.setMaxNALocals(this.code_max_na_locals.getInt());
            code.setHandlerCount(this.code_handler_count.getInt());
        }
        this.code_max_stack.doneDisbursing();
        this.code_max_na_locals.doneDisbursing();
        this.code_handler_count.doneDisbursing();
        this.readCodeHandlers();
        if (!(testBit == 0)) {
            this.codesWithFlags = Arrays.asList(this.allCodes);
        }
        else {
            this.codesWithFlags = codesWithFlags;
        }
        this.countAttrs(3, this.codesWithFlags);
    }
    
    void readCodeHandlers() throws IOException {
        int n = 0;
        for (int i = 0; i < this.allCodes.length; ++i) {
            n += this.allCodes[i].getHandlerCount();
        }
        BandStructure.ValueBand[] array = { this.code_handler_start_P, this.code_handler_end_PO, this.code_handler_catch_PO, this.code_handler_class_RCN };
        for (int j = 0; j < array.length; ++j) {
            array[j].expectLength(n);
            array[j].readFrom(this.in);
        }
        for (int k = 0; k < this.allCodes.length; ++k) {
            Code code = this.allCodes[k];
            for (int l = 0; l < code.getHandlerCount(); ++l) {
                code.handler_class[l] = this.code_handler_class_RCN.getRef();
                code.handler_start[l] = this.code_handler_start_P.getInt();
                code.handler_end[l] = this.code_handler_end_PO.getInt();
                code.handler_catch[l] = this.code_handler_catch_PO.getInt();
            }
        }
        for (int n2 = 0; n2 < array.length; ++n2) {
            array[n2].doneDisbursing();
        }
    }
    
    void fixupCodeHandlers() {
        for (int i = 0; i < this.allCodes.length; ++i) {
            Code code = this.allCodes[i];
            for (int j = 0; j < code.getHandlerCount(); ++j) {
                int n = code.handler_start[j];
                code.handler_start[j] = code.decodeBCI(n);
                int n2 = n + code.handler_end[j];
                code.handler_end[j] = code.decodeBCI(n2);
                code.handler_catch[j] = code.decodeBCI(n2 + code.handler_catch[j]);
            }
        }
    }
    
    void countAndReadAttrs(int n, Collection<? extends Attribute.Holder> collection) throws IOException {
        this.countAttrs(n, collection);
        this.readAttrs(n, collection);
    }
    
    void countAttrs(int n, Collection<? extends Attribute.Holder> collection) throws IOException {
        BandStructure.MultiBand multiBand = this.attrBands[n];
        long n2 = this.attrFlagMask[n];
        if (this.verbose > 1) {
            Utils.log.fine("scanning flags and attrs for " + Attribute.contextName(n) + "[" + collection.size() + "]");
        }
        List<Object> list = (List)this.attrDefs.get(n);
        Attribute.Layout[] array = new Attribute.Layout[list.size()];
        list.toArray(array);
        BandStructure.IntBand attrBand = BandStructure.getAttrBand(multiBand, 0);
        BandStructure.IntBand attrBand2 = BandStructure.getAttrBand(multiBand, 1);
        BandStructure.IntBand attrBand3 = BandStructure.getAttrBand(multiBand, 2);
        BandStructure.IntBand attrBand4 = BandStructure.getAttrBand(multiBand, 3);
        BandStructure.IntBand attrBand5 = BandStructure.getAttrBand(multiBand, 4);
        int n3 = this.attrOverflowMask[n];
        int n4 = 0;
        int haveFlagsHi = this.haveFlagsHi(n);
        attrBand.expectLength(haveFlagsHi ? collection.size() : 0);
        attrBand.readFrom(this.in);
        attrBand2.expectLength(collection.size());
        attrBand2.readFrom(this.in);
        assert (n2 & (long)n3) == (long)n3;
        for (Attribute.Holder holder : collection) {
            int int1 = attrBand2.getInt();
            holder.flags = int1;
            if ((int1 & n3) != 0x0) {
                ++n4;
            }
            else {
                continue;
            }
        }
        attrBand3.expectLength(n4);
        attrBand3.readFrom(this.in);
        attrBand4.expectLength(attrBand3.getIntTotal());
        attrBand4.readFrom(this.in);
        int[] array2 = new int[array.length];
        for (Attribute.Holder holder2 : collection) {
            assert holder2.attributes == null;
            long n5 = ((long)holder2.flags & n2) << 32 >>> 32;
            holder2.flags -= (int)n5;
            assert holder2.flags == (char)holder2.flags;
            assert holder2.flags == 0;
            if (!(haveFlagsHi == 0)) {
                n5 += (long)attrBand.getInt() << 32;
            }
            if (n5 == 0L) {
                continue;
            }
            int i = 0;
            long n6 = n5 & (long)n3;
            assert n6 >= 0L;
            long n7 = n5 - n6;
            if (n6 != 0L) {
                i = attrBand3.getInt();
            }
            int n8 = 0;
            for (int n9 = 0; n7 != 0L; ++n9) {
                if ((n7 & 1L << n9) != 0x0L) {
                    n7 -= 1L << n9;
                    ++n8;
                }
            }
            ArrayList attributes = new ArrayList(n8 + i);
            holder2.attributes = attributes;
            for (int n10 = 0; n7 != 0L; ++n10) {
                if ((n7 & 1L << n10) != 0x0L) {
                    n7 -= 1L << n10;
                    int n11 = n10;
                    ++array2[n11];
                    if (array[n10] == null) {
                        this.badAttrIndex(n10, n);
                    }
                    attributes.add(array[n10].canonicalInstance());
                    --n8;
                }
            }
            assert n8 == 0;
            while (i > 0) {
                int int2;
                int n12 = int2 = attrBand4.getInt();
                ++array2[int2];
                if (array[n12] == null) {
                    this.badAttrIndex(n12, n);
                }
                attributes.add(array[n12].canonicalInstance());
                --i;
            }
        }
        attrBand.doneDisbursing();
        attrBand2.doneDisbursing();
        attrBand3.doneDisbursing();
        attrBand4.doneDisbursing();
        int n13 = 0;
        boolean n14 = true;
        while (true) {
            for (int j = 0; j < array.length; ++j) {
                Attribute.Layout layout = array[j];
                if (layout != null && n14 == (this.isPredefinedAttr(n, j) ? 1 : 0) && array2[j] != 0) {
                    Attribute.Layout.Element[] callables = layout.getCallables();
                    int k = 0;
                    while (k < callables.length) {
                        assert callables[k].kind == 10;
                        if (callables[k].flagTest((byte)8)) {
                            ++n13;
                        }
                        ++k;
                    }
                }
            }
            if (n14 == 0) {
                attrBand5.expectLength(n13);
                attrBand5.readFrom(this.in);
                boolean n15 = true;
                while (true) {
                    for (int l = 0; l < array.length; ++l) {
                        Attribute.Layout layout2 = array[l];
                        if (layout2 != null && n15 == (this.isPredefinedAttr(n, l) ? 1 : 0)) {
                            int n16 = array2[l];
                            BandStructure.Band[] array3 = (BandStructure.Band[])this.attrBandTable.get(layout2);
                            if (layout2 == this.attrInnerClassesEmpty) {
                                this.class_InnerClasses_N.expectLength(n16);
                                this.class_InnerClasses_N.readFrom(this.in);
                                int intTotal = this.class_InnerClasses_N.getIntTotal();
                                this.class_InnerClasses_RC.expectLength(intTotal);
                                this.class_InnerClasses_RC.readFrom(this.in);
                                this.class_InnerClasses_F.expectLength(intTotal);
                                this.class_InnerClasses_F.readFrom(this.in);
                                int n21 = intTotal - this.class_InnerClasses_F.getIntCount(0);
                                this.class_InnerClasses_outer_RCN.expectLength(n21);
                                this.class_InnerClasses_outer_RCN.readFrom(this.in);
                                this.class_InnerClasses_name_RUN.expectLength(n21);
                                this.class_InnerClasses_name_RUN.readFrom(this.in);
                            }
                            else if (!this.optDebugBands && n16 == 0) {
                                for (int n20 = 0; n20 < array3.length; ++n20) {
                                    array3[n20].doneWithUnusedBand();
                                }
                            }
                            else {
                                if (!layout2.hasCallables()) {
                                    this.readAttrBands(layout2.elems, n16, new int[0], array3);
                                }
                                else {
                                    Attribute.Layout.Element[] callables2 = layout2.getCallables();
                                    int[] array4 = new int[callables2.length];
                                    array4[0] = n16;
                                    int n17 = 0;
                                    while (n17 < callables2.length) {
                                        assert callables2[n17].kind == 10;
                                        int n18 = array4[n17];
                                        array4[n17] = -1;
                                        if (n16 > 0 && callables2[n17].flagTest((byte)8)) {
                                            n18 += attrBand5.getInt();
                                        }
                                        this.readAttrBands(callables2[n17].body, n18, array4, array3);
                                        ++n17;
                                    }
                                }
                                if (this.optDebugBands && n16 == 0) {
                                    for (int n19 = 0; n19 < array3.length; ++n19) {
                                        array3[n19].doneDisbursing();
                                    }
                                }
                            }
                        }
                    }
                    if (n15 == 0) {
                        attrBand5.doneDisbursing();
                        return;
                    }
                    n15 = false;
                }
            }
            else {
                n14 = false;
            }
        }
    }
    
    void badAttrIndex(int i, int n) throws IOException {
        throw new IOException("Unknown attribute index " + i + " for " + Constants.ATTR_CONTEXT_NAME[n] + " attribute");
    }
    
    void readAttrs(int n, Collection<? extends Attribute.Holder> collection) throws IOException {
        HashSet<Attribute.Layout> set = new HashSet();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        for (Attribute.Holder obj : collection) {
            if (obj.attributes == null) {
                continue;
            }
            ListIterator<Attribute> listIterator = obj.attributes.listIterator();
            while (listIterator.hasNext()) {
                Attribute obj2 = (Attribute)listIterator.next();
                Attribute.Layout layout = obj2.layout();
                if (layout.bandCount == 0) {
                    if (layout != this.attrInnerClassesEmpty) {
                        continue;
                    }
                    this.readLocalInnerClasses((Package.Class)obj);
                }
                else {
                    set.add(layout);
                    int n2 = n == 1 && layout == this.attrConstantValue;
                    if (!(n2 == 0)) {
                        this.setConstantValueIndex((Package.Class.Field)obj);
                    }
                    if (this.verbose > 2) {
                        Utils.log.fine("read " + obj2 + " in " + obj);
                    }
                    final BandStructure.Band[] array = (BandStructure.Band[])this.attrBandTable.get(layout);
                    byteArrayOutputStream.reset();
                    listIterator.set(obj2.addContent(byteArrayOutputStream.toByteArray(), obj2.unparse(new Attribute.ValueStream() {
                        @Override
                        public int getInt(int n) {
                            return ((BandStructure.IntBand)array[n]).getInt();
                        }
                        
                        @Override
                        public ConstantPool.Entry getRef(int n) {
                            return ((BandStructure.CPRefBand)array[n]).getRef();
                        }
                        
                        @Override
                        public int decodeBCI(int n) {
                            return ((Code)obj).decodeBCI(n);
                        }
                    }, byteArrayOutputStream)));
                    if (!(n2 == 0)) {
                        this.setConstantValueIndex(null);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        for (Attribute.Layout layout2 : set) {
            if (layout2 == null) {
                continue;
            }
            BandStructure.Band[] array2 = (BandStructure.Band[])this.attrBandTable.get(layout2);
            for (int i = 0; i < array2.length; ++i) {
                array2[i].doneDisbursing();
            }
        }
        if (n == 0) {
            this.class_InnerClasses_N.doneDisbursing();
            this.class_InnerClasses_RC.doneDisbursing();
            this.class_InnerClasses_F.doneDisbursing();
            this.class_InnerClasses_outer_RCN.doneDisbursing();
            this.class_InnerClasses_name_RUN.doneDisbursing();
        }
        BandStructure.MultiBand multiBand = this.attrBands[n];
        for (int j = 0; j < multiBand.size(); ++j) {
            BandStructure.Band value = multiBand.get(j);
            if ((value instanceof BandStructure.MultiBand)) {
                value.doneDisbursing();
            }
        }
        multiBand.doneDisbursing();
    }
    
    private void readAttrBands(Attribute.Layout.Element[] array, int n, int[] array2, BandStructure.Band[] array3) throws IOException {
        for (int i = 0; i < array.length; ++i) {
            Attribute.Layout.Element element = array[i];
            BandStructure.Band band = null;
            if (element.hasBand()) {
                band = array3[element.bandIndex];
                band.expectLength(n);
                band.readFrom(this.in);
            }
            switch (element.kind) {
                case 5: {
                    this.readAttrBands(element.body, ((BandStructure.IntBand)band).getIntTotal(), array2, array3);
                    break;
                }
                case 7: {
                    for (int j = 0; j < element.body.length; ++j) {
                        if (j == element.body.length - 1) {
                        }
                        else {
                            n = 0;
                            for (; j == j || (j < element.body.length && element.body[j].flagTest((byte)8)); ++j) {
                                n += ((BandStructure.IntBand)band).getIntCount(element.body[j].value);
                            }
                            --j;
                        }
                        n -= n;
                        this.readAttrBands(element.body[j].body, n, array2, array3);
                    }
                    assert n == 0;
                    break;
                }
                case 9: {
                    assert element.body.length == 1;
                    assert element.body[0].kind == 10;
                    if (!element.flagTest((byte)8)) {
                        assert array2[element.value] >= 0;
                        int value = element.value;
                        array2[value] += n;
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 10: {
                    assert false;
                    break;
                }
            }
        }
    }
    
    void readByteCodes() throws IOException {
        this.bc_codes.elementCountForDebug = this.allCodes.length;
        this.bc_codes.setInputStreamFrom(this.in);
        this.readByteCodeOps();
        this.bc_codes.doneDisbursing();
        BandStructure.Band[] array = { this.bc_case_value, this.bc_byte, this.bc_short, this.bc_local, this.bc_label, this.bc_intref, this.bc_floatref, this.bc_longref, this.bc_doubleref, this.bc_stringref, this.bc_loadablevalueref, this.bc_classref, this.bc_fieldref, this.bc_methodref, this.bc_imethodref, this.bc_indyref, this.bc_thisfield, this.bc_superfield, this.bc_thismethod, this.bc_supermethod, this.bc_initref, this.bc_escref, this.bc_escrefsize, this.bc_escsize };
        for (int i = 0; i < array.length; ++i) {
            array[i].readFrom(this.in);
        }
        this.bc_escbyte.expectLength(this.bc_escsize.getIntTotal());
        this.bc_escbyte.readFrom(this.in);
        this.expandByteCodeOps();
        this.bc_case_count.doneDisbursing();
        for (int j = 0; j < array.length; ++j) {
            array[j].doneDisbursing();
        }
        this.bc_escbyte.doneDisbursing();
        this.bc_bands.doneDisbursing();
        this.readAttrs(3, this.codesWithFlags);
        this.fixupCodeHandlers();
        this.code_bands.doneDisbursing();
        this.class_bands.doneDisbursing();
    }
    
    private void readByteCodeOps() throws IOException {
        byte[] realloc = new byte[4096];
        ArrayList<Integer> list = new ArrayList();
        int i = 0;
    Label_0016:
        while (i < this.allCodes.length) {
            Code code = this.allCodes[i];
            int n = 0;
            while (true) {
                int j = this.bc_codes.getByte();
                if (n + 10 > realloc.length) {
                    realloc = BandStructure.realloc(realloc);
                }
                realloc[n] = (byte)j;
                boolean n2 = false;
                if (j == 196) {
                    j = this.bc_codes.getByte();
                    realloc[++n] = (byte)j;
                    n2 = true;
                }
                assert j == (0xFF & j);
                switch (j) {
                    case 170:
                    case 171: {
                        this.bc_case_count.expectMoreLength(1);
                        list.add(Integer.valueOf(j));
                        break;
                    }
                    case 132: {
                        this.bc_local.expectMoreLength(1);
                        if (!(n2 == 0)) {
                            this.bc_short.expectMoreLength(1);
                            break;
                        }
                        this.bc_byte.expectMoreLength(1);
                        break;
                    }
                    case 17: {
                        this.bc_short.expectMoreLength(1);
                        break;
                    }
                    case 16: {
                        this.bc_byte.expectMoreLength(1);
                        break;
                    }
                    case 188: {
                        this.bc_byte.expectMoreLength(1);
                        break;
                    }
                    case 197: {
                        assert this.getCPRefOpBand(j) == this.bc_classref;
                        this.bc_classref.expectMoreLength(1);
                        this.bc_byte.expectMoreLength(1);
                        break;
                    }
                    case 253: {
                        this.bc_escrefsize.expectMoreLength(1);
                        this.bc_escref.expectMoreLength(1);
                        break;
                    }
                    case 254: {
                        this.bc_escsize.expectMoreLength(1);
                        break;
                    }
                    case 255: {
                        code.bytes = BandStructure.realloc(realloc, n);
                        ++i;
                        continue Label_0016;
                    }
                    default: {
                        if (Instruction.isInvokeInitOp(j)) {
                            this.bc_initref.expectMoreLength(1);
                            break;
                        }
                        if (Instruction.isSelfLinkerOp(j)) {
                            this.selfOpRefBand(j).expectMoreLength(1);
                            break;
                        }
                        if (Instruction.isBranchOp(j)) {
                            this.bc_label.expectMoreLength(1);
                            break;
                        }
                        if (Instruction.isCPRefOp(j)) {
                            this.getCPRefOpBand(j).expectMoreLength(1);
                            assert j != 197;
                            break;
                        }
                        else {
                            if (Instruction.isLocalSlotOp(j)) {
                                this.bc_local.expectMoreLength(1);
                                break;
                            }
                            break;
                        }
                    }
                }
                ++n;
            }
        }
        this.bc_case_count.readFrom(this.in);
        for (int intValue : list) {
            int int1 = this.bc_case_count.getInt();
            this.bc_label.expectMoreLength(1 + int1);
            this.bc_case_value.expectMoreLength((intValue == 170) ? 1 : int1);
        }
        this.bc_case_count.resetForSecondPass();
    }
    
    private void expandByteCodeOps() throws IOException {
        byte[] a = new byte[4096];
        int[] realloc = new int[4096];
        int[] realloc2 = new int[1024];
        Fixups obj = new Fixups();
        for (int i = 0; i < this.allCodes.length; ++i) {
            Code code = this.allCodes[i];
            byte[] bytes = code.bytes;
            code.bytes = null;
            Package.Class thisClass = code.thisClass();
            Set<ConstantPool.Entry> set = (Set<ConstantPool.Entry>)this.ldcRefMap.get(thisClass);
            if (set == null) {
                this.ldcRefMap.put(thisClass, set = new HashSet());
            }
            ConstantPool.ClassEntry thisClass2 = thisClass.thisClass;
            ConstantPool.ClassEntry superClass = thisClass.superClass;
            ConstantPool.ClassEntry classEntry = null;
            int j = 0;
            int n = 0;
            int n2 = 0;
            obj.clear();
            for (int k = 0; k < bytes.length; ++k) {
                int l = Instruction.getByte(bytes, k);
                realloc[n++] = j;
                if (j + 10 > a.length) {
                    a = BandStructure.realloc(a);
                }
                if (n + 10 > realloc.length) {
                    realloc = BandStructure.realloc(realloc);
                }
                if (n2 + 10 > realloc2.length) {
                    realloc2 = BandStructure.realloc(realloc2);
                }
                boolean n3 = false;
                if (l == 196) {
                    a[j++] = (byte)l;
                    l = Instruction.getByte(bytes, ++k);
                    n3 = true;
                }
                switch (l) {
                    case 170:
                    case 171: {
                        int int1;
                        for (int1 = this.bc_case_count.getInt(); j + 30 + int1 * 8 > a.length; a = BandStructure.realloc(a)) {}
                        a[j++] = (byte)l;
                        Arrays.fill(a, j, j + 30, (byte)0);
                        Instruction.Switch switch1 = (Instruction.Switch)Instruction.at(a, j);
                        switch1.setCaseCount(int1);
                        if (l == 170) {
                            switch1.setCaseValue(0, this.bc_case_value.getInt());
                        }
                        else {
                            for (int n4 = 0; n4 < int1; ++n4) {
                                switch1.setCaseValue(n4, this.bc_case_value.getInt());
                            }
                        }
                        realloc2[n2++] = j;
                        j = switch1.getNextPC();
                        break;
                    }
                    case 132: {
                        a[j++] = (byte)l;
                        int int2 = this.bc_local.getInt();
                        if (!(n3 == 0)) {
                            int int3 = this.bc_short.getInt();
                            Instruction.setShort(a, j, int2);
                            j += 2;
                            Instruction.setShort(a, j, int3);
                            j += 2;
                            break;
                        }
                        int n5 = (byte)this.bc_byte.getByte();
                        a[j++] = (byte)int2;
                        a[j++] = (byte)n5;
                        break;
                    }
                    case 17: {
                        int int4 = this.bc_short.getInt();
                        a[j++] = (byte)l;
                        Instruction.setShort(a, j, int4);
                        j += 2;
                        break;
                    }
                    case 16:
                    case 188: {
                        int byte1 = this.bc_byte.getByte();
                        a[j++] = (byte)l;
                        a[j++] = (byte)byte1;
                        break;
                    }
                    case 253: {
                        int int5 = this.bc_escrefsize.getInt();
                        ConstantPool.Entry ref = this.bc_escref.getRef();
                        if (int5 == 1) {
                            set.add(ref);
                        }
                        switch (int5) {
                            case 1: {
                                obj.addU1(j, ref);
                                break;
                            }
                            case 2: {
                                obj.addU2(j, ref);
                                break;
                            }
                            default: {
                                assert false;
                                break;
                            }
                        }
                        a[j + 0] = (a[j + 1] = 0);
                        j += int5;
                        break;
                    }
                    case 254: {
                        int int6;
                        for (int6 = this.bc_escsize.getInt(); j + int6 > a.length; a = BandStructure.realloc(a)) {}
                        while (int6-- > 0) {
                            a[j++] = (byte)this.bc_escbyte.getByte();
                        }
                        break;
                    }
                    default: {
                        if (Instruction.isInvokeInitOp(l)) {
                            int n12 = l - 230;
                            int n13 = 183;
                            switch (n12) {
                                case 0: {
                                    break;
                                }
                                case 1: {
                                    thisClass2 = superClass;
                                    break;
                                }
                                default: {
                                    assert n12 == 2;
                                    thisClass2 = classEntry;
                                    break;
                                }
                            }
                            a[j++] = (byte)n13;
                            obj.addU2(j, this.pkg.cp.getOverloadingForIndex((byte)10, thisClass2, "<init>", this.bc_initref.getInt()));
                            a[j + 0] = (a[j + 1] = 0);
                            j += 2;
                            assert Instruction.opLength(n13) == j - j;
                            break;
                        }
                        else if (Instruction.isSelfLinkerOp(l)) {
                            int n8 = l - 202;
                            int n9 = n8 >= 14;
                            if (!(n9 == 0)) {
                                n8 -= 14;
                            }
                            int n10 = n8 >= 7;
                            if (!(n10 == 0)) {
                                n8 -= 7;
                            }
                            int n11 = 178 + n8;
                            int fieldOp = Instruction.isFieldOp(n11);
                            thisClass2 = (n9 ? superClass : thisClass2);
                            BandStructure.CPRefBand cpRefBand;
                            ConstantPool.Index index;
                            if (!(fieldOp == 0)) {
                                cpRefBand = (n9 ? this.bc_superfield : this.bc_thisfield);
                                index = this.pkg.cp.getMemberIndex((byte)9, thisClass2);
                            }
                            else {
                                cpRefBand = (n9 ? this.bc_supermethod : this.bc_thismethod);
                                index = this.pkg.cp.getMemberIndex((byte)10, thisClass2);
                            }
                            assert cpRefBand == this.selfOpRefBand(l);
                            ConstantPool.MemberEntry memberEntry = (ConstantPool.MemberEntry)cpRefBand.getRef(index);
                            if (!(n10 == 0)) {
                                a[j++] = 42;
                                realloc[n++] = j;
                            }
                            a[j++] = (byte)n11;
                            obj.addU2(j, memberEntry);
                            a[j + 0] = (a[j + 1] = 0);
                            j += 2;
                            assert Instruction.opLength(n11) == j - j;
                            break;
                        }
                        else if (Instruction.isBranchOp(l)) {
                            a[j++] = (byte)l;
                            assert n3 == 0;
                            int n7 = j + Instruction.opLength(l);
                            realloc2[n2++] = j;
                            while (j < n7) {
                                a[j++] = 0;
                            }
                            break;
                        }
                        else if (Instruction.isCPRefOp(l)) {
                            BandStructure.CPRefBand cpRefOpBand = this.getCPRefOpBand(l);
                            ConstantPool.Entry ref2 = cpRefOpBand.getRef();
                            if (ref2 == null) {
                                if (cpRefOpBand == this.bc_classref) {
                                    ref2 = thisClass2;
                                }
                                else
                                    assert false;
                            }
                            int n6 = 2;
                            switch (l) {
                                case 243: {
                                    l = 184;
                                    break;
                                }
                                case 242: {
                                    l = 183;
                                    break;
                                }
                                case 18:
                                case 233:
                                case 234:
                                case 235:
                                case 240: {
                                    l = 18;
                                    n6 = 1;
                                    set.add(ref2);
                                    break;
                                }
                                case 19:
                                case 236:
                                case 237:
                                case 238:
                                case 241: {
                                    l = 19;
                                    break;
                                }
                                case 20:
                                case 239: {
                                    l = 20;
                                    break;
                                }
                                case 187: {
                                    classEntry = (ConstantPool.ClassEntry)ref2;
                                    break;
                                }
                            }
                            a[j++] = (byte)l;
                            switch (n6) {
                                case 1: {
                                    obj.addU1(j, ref2);
                                    break;
                                }
                                case 2: {
                                    obj.addU2(j, ref2);
                                    break;
                                }
                                default: {
                                    assert false;
                                    break;
                                }
                            }
                            a[j + 0] = (a[j + 1] = 0);
                            j += n6;
                            if (l == 197) {
                                a[j++] = (byte)this.bc_byte.getByte();
                            }
                            else if (l == 185) {
                                a[j++] = (byte)(1 + ((ConstantPool.MemberEntry)ref2).descRef.typeRef.computeSize(true));
                                a[j++] = 0;
                            }
                            else if (l == 186) {
                                a[j++] = 0;
                                a[j++] = 0;
                            }
                            assert Instruction.opLength(l) == j - j;
                            break;
                        }
                        else if (Instruction.isLocalSlotOp(l)) {
                            a[j++] = (byte)l;
                            int int7 = this.bc_local.getInt();
                            if (!(n3 == 0)) {
                                Instruction.setShort(a, j, int7);
                                j += 2;
                                if (l == 132) {
                                    Instruction.setShort(a, j, this.bc_short.getInt());
                                    j += 2;
                                }
                            }
                            else {
                                Instruction.setByte(a, j, int7);
                                ++j;
                                if (l == 132) {
                                    Instruction.setByte(a, j, this.bc_byte.getByte());
                                    ++j;
                                }
                            }
                            assert Instruction.opLength(l) == j - j;
                            break;
                        }
                        else {
                            if (l >= 202) {
                                Utils.log.warning("unrecognized bytescode " + l + " " + Instruction.byteName(l));
                            }
                            assert l < 202;
                            a[j++] = (byte)l;
                            assert Instruction.opLength(l) == j - j;
                            break;
                        }
                    }
                }
            }
            code.setBytes(BandStructure.realloc(a, j));
            code.setInstructionMap(realloc, n);
            Instruction at = null;
            for (int n15 : realloc2) {
                at = Instruction.at(code.bytes, n15, at);
                if ((at instanceof Instruction.Switch)) {
                    Instruction.Switch switch2 = (Instruction.Switch)at;
                    switch2.setDefaultLabel(this.getLabel(this.bc_label, code, n15));
                    for (int caseCount = switch2.getCaseCount(), n16 = 0; n16 < caseCount; ++n16) {
                        switch2.setCaseLabel(n16, this.getLabel(this.bc_label, code, n15));
                    }
                }
                else {
                    at.setBranchLabel(this.getLabel(this.bc_label, code, n15));
                }
            }
            if (obj.size() > 0) {
                if (this.verbose > 2) {
                    Utils.log.fine("Fixups in code: " + obj);
                }
                code.addFixups(obj);
            }
        }
    }
    
    static int access$000(PackageReader packageReader, ConstantPool.Entry entry) {
        return packageReader.getOutputIndex(entry);
    }
    
    static class LimitedBuffer extends BufferedInputStream
    {
        long served;
        int servedPos;
        long limit;
        long buffered;
        
        public boolean atLimit() {
            int n = this.getBytesServed() == this.limit;
            assert this.limit == this.buffered;
            return n;
        }
        
        public long getBytesServed() {
            return this.served + (long)(this.pos - this.servedPos);
        }
        
        public void setReadLimit(long n) {
            if (n == -1L) {
                this.limit = -1L;
            }
            else {
                this.limit = this.getBytesServed() + n;
            }
        }
        
        public long getReadLimit() {
            if (this.limit == -1L) {
                return this.limit;
            }
            return this.limit - this.getBytesServed();
        }
        
        @Override
        public int read() throws IOException {
            if (this.pos < this.count) {
                return this.buf[this.pos++] & 0xFF;
            }
            this.served += (long)(this.pos - this.servedPos);
            int read = super.read();
            this.servedPos = this.pos;
            if (read >= 0) {
                ++this.served;
            }
            assert this.limit == -1L;
            return read;
        }
        
        @Override
        public int read(byte[] b, int off, int len) throws IOException {
            this.served += (long)(this.pos - this.servedPos);
            int read = super.read(b, off, len);
            this.servedPos = this.pos;
            if (read >= 0) {
                this.served += (long)read;
            }
            return read;
        }
        
        @Override
        public long skip(long n) throws IOException {
            throw new RuntimeException("no skipping");
        }
        
        LimitedBuffer(InputStream inputStream) {
            super(null, 16384);
            this.servedPos = this.pos;
            super.in = new FilterInputStream(inputStream) {
                @Override
                public int read() throws IOException {
                    if (PackageReader.LimitedBuffer.this.buffered == PackageReader.LimitedBuffer.this.limit) {
                        return -1;
                    }
                    PackageReader.LimitedBuffer this$0 = PackageReader.LimitedBuffer.this;
                    ++this$0.buffered;
                    return super.read();
                }
                
                @Override
                public int read(byte[] b, int off, int len) throws IOException {
                    if (PackageReader.LimitedBuffer.this.buffered == PackageReader.LimitedBuffer.this.limit) {
                        return -1;
                    }
                    if (PackageReader.LimitedBuffer.this.limit != -1L) {
                        long n = PackageReader.LimitedBuffer.this.limit - PackageReader.LimitedBuffer.this.buffered;
                        if ((long)len > n) {
                            len = (int)n;
                        }
                    }
                    int read = super.read(b, off, len);
                    if (read >= 0) {
                        PackageReader.LimitedBuffer this$0 = PackageReader.LimitedBuffer.this;
                        this$0.buffered += (long)read;
                    }
                    return read;
                }
            };
        }
    }
}
